
#ifndef WASM_RELOC
#error "WASM_RELOC must be defined"
#endif

WASM_RELOC(R_WEBASSEMBLY_FUNCTION_INDEX_LEB,   0)
WASM_RELOC(R_WEBASSEMBLY_TABLE_INDEX_SLEB,     1)
WASM_RELOC(R_WEBASSEMBLY_TABLE_INDEX_I32,      2)
WASM_RELOC(R_WEBASSEMBLY_MEMORY_ADDR_LEB,      3)
WASM_RELOC(R_WEBASSEMBLY_MEMORY_ADDR_SLEB,     4)
WASM_RELOC(R_WEBASSEMBLY_MEMORY_ADDR_I32,      5)
WASM_RELOC(R_WEBASSEMBLY_TYPE_INDEX_LEB,       6)
WASM_RELOC(R_WEBASSEMBLY_GLOBAL_INDEX_LEB,     7)
