



if not exist %LIBRARY_INC%\\llvm\\Pass.h exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %LIBRARY_LIB%\\LLVMSupport.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
opt -S -vector-library=SVML -mcpu=haswell -O3 numba-3016.ll -o out.ll
IF %ERRORLEVEL% NEQ 0 exit /B 1
findstr __svml_sin4 out.ll
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
